@@ -9,9 +9,9 @@
 #define BFS_EXPR_H
 
 #include "color.h"
+#include "config.h"
 #include "eval.h"
 #include "stat.h"
-#include <stdbool.h>
 #include <stddef.h>
 #include <sys/types.h>
 #include <time.h>
@@ -143,6 +143,16 @@ struct bfs_expr {
 			};
 		};
 
+		/** String comparisons. */
+		struct {
+			/** String pattern. */
+			const char *pattern;
+			/** fnmatch() flags. */
+			int fnm_flags;
+			/** Whether strcmp() can be used instead of fnmatch(). */
+			bool literal;
+		};
+
 		/** Printing actions. */
 		struct {
 			/** The output stream. */
